function errors = measureErrors()
%measureErrors Measure intrinsic image errors and temporal incoherence
%compared to ground truth.
%   By default, the ground truth data file is assumed to be under 
%   [loc dir_data_gt testnames{i} '/data.mat']
%   , and result file(s) under
%   [loc dir_data_res testnames{i} '/result.mat']
%   ... (optionally more .mat files in the format below).
% 
% Format of 'data.mat'
% - images	: input video volume (height x width x channels x frames)
% - albedos_gt	: ground truth albedo sequence (height x width x channels x frames)
% - illumis_gt	: ground truth shading sequence (height x width x channels x frames)
% - flows_gt	: ground truth optical flow sequence (height x width x 2 x frames-1)
% - occlus_gt	: ground truth occlusion map sequence (height x width x 1 x frames-1)
% 
% Format of 'result.mat'
% - albedos_est	: estimated albedo sequence (height x width x channels x frames)
% - illumis_est	: estimated shading sequence (height x width x channels x frames)
% - flows_est_brt: flow estimated from 'albedos_est' (height x width x 2 x frames-1)
% 
% Format of 'errors'
% 'k' below is an index of result(s) to measure errors (see line 56). 
%   .intr(k)    : intrinsic image error. aka LMSE
%     .sse      : LMSEx1000 from a mean albedo image and a single-channel shading image
%     .sse_cmean: mean LMSEx1000 by averaging each channel's LMSE [used in the paper]
%   .temp(k)    : temporal incoherence of an albedo sequence. aka EPE
%     .aepe     : EPE including the whole image region
%     .aepe_mask: EPE excluding ground truth occluded regions
% 
% NOTE: '.sse' above uses only Channel #1 of 'illumis_gt' and 'illumis_est'.
% 
% 
% Copyright (c) 2014, Naejin Kong, Max Planck Institute for Intelligent 
% Systems, Tuebingen, Germany
% 
%                          All Rights Reserved.
% 
% Contact: naejin.kong@tuebingen.mpg.de

loc = '../';

dir_data_gt  = 'data/';
dir_data_res = 'results/';

testnames = {'data_synth1', 'data_synth2', 'data_synth3'};

paths_gt  = cell(1,length(testnames));
paths_mys = cell(1,length(testnames));
for i=1:length(testnames),
    paths_gt{i}     = [loc dir_data_gt testnames{i} '/data.mat'];  
    
    paths_mys{i}    = { ...
        [loc dir_data_res testnames{i} '/result.mat'] ...         % result 1 (k=1)
%       [loc dir_data_res testnames{i} '/result_method2.mat'] ... % result 2 (k=2)
%       [loc dir_data_res testnames{i} '/result_method3.mat'] ... % result 3 (k=3)
    };
end

addpath('utils/');

%% intrinsic image error measure (aka LMSE)
if 1,
    
    % sse: get an LMSE for each image where albedo is averaged over RGB channels, 
    %      and average the LMSEs over all frames.
    % sse_cmean: get an LMSE at each channel, average them over RGB channels, 
    %      and average them over all frames.
    errors_intr_mean_all_my = repmat(struct('sse',[],'sse_cmean',[]), [length(paths_mys{1}) length(paths_gt)]);
    for i = 1:length(paths_gt),
        errors_mean_my_tmp = measureIntr( paths_gt{i}, paths_mys{i} );

        for k=1:length(errors_mean_my_tmp),
            errors_intr_mean_all_my(k,i) = errors_mean_my_tmp(k);
        end
    end
    
    errors.intr = repmat(struct('sse',[],'sse_cmean',[]), [length(paths_mys{1}) 1]);

    % get mean of LMSEs over all examples
    for k=1:size(errors_intr_mean_all_my,1),
        errors.intr(k).sse = mean([errors_intr_mean_all_my(k,:).sse]);
        errors.intr(k).sse_cmean = mean([errors_intr_mean_all_my(k,:).sse_cmean]);
        
        fprintf('** [result %d] mean LMSE x 1000 over all examples : LMSE = %3.5f, LMSE(Ch-Mean) = %3.5f\n\n', ...
            k, errors.intr(k).sse, errors.intr(k).sse_cmean);
    end
end

%% temporal incoherence measure (aka EPE)
if 1,
    
    errors_flow_mean_all_my = repmat(struct('aepe',[],'aepe_mask',[]), [length(paths_mys{1}) length(paths_gt)]);    
    for i = 1:length(paths_gt),    
        errors_mean_my_tmp = measureFlow( paths_gt{i}, paths_mys{i} );

        for k=1:length(errors_mean_my_tmp),
            errors_flow_mean_all_my(k,i) = errors_mean_my_tmp(k);
        end

    end
    
    errors.temp = repmat(struct('aepe',[],'aepe_mask',[]), [length(paths_mys{1}) 1]);
    
    for k=1:size(errors_flow_mean_all_my,1),
        errors.temp(k).aepe = mean([errors_flow_mean_all_my(k,:).aepe]);
        errors.temp(k).aepe_mask = mean([errors_flow_mean_all_my(k,:).aepe_mask]);
        
        fprintf('** [result %d] mean EPE over all examples: EPE = %3.5f, EPE excl. occlusion = %3.5f\n\n', ...
            k, errors.temp(k).aepe, errors.temp(k).aepe_mask);
    end
end


% -------- local functions ----------

%% mean of LMSEs over all examples

    function [errors_mean, errors] = measureIntr( path_gt, paths_my )
        windowSz = 20;
        
        gtdata = load(path_gt);
        gtdata.albedos_gt = double(gtdata.albedos_gt);
        gtdata.illumis_gt = double(gtdata.illumis_gt);
        
        fprintf('--- Intrinsic: [gt] %s\n', path_gt);

        errors_mean = [];
        for ii=1:length(paths_my),
            
            fprintf('[result %d] %s\n', ii, paths_my{ii});
            
            estdata = load(paths_my{ii});
            estdata.albedos_est = double(estdata.albedos_est);
            estdata.illumis_est = double(estdata.illumis_est);        

            [avg_sse, avg_sse_cmean, errors] = calcLMSEs(estdata, gtdata, windowSz);
            
            errors_mean(end+1).sse = avg_sse*1000;
            errors_mean(end).sse_cmean = avg_sse_cmean*1000;        
        end        
        
    end

%% LMSE averaged over all frames

    function [avg_sse, avg_sse_cmean, errors] = calcLMSEs(estdata, gtdata, windowSz)
        if nargin < 3,
            windowSz = 20;
        end
        
        nframes = size(estdata.albedos_est,4);
        channels = size(estdata.albedos_est,3);
        
        errors = repmat(struct('sse',nan,'sse_cmean',nan), [nframes 1]);
        for tt=1:nframes,
            % LMSE whose albedo image is averaged over RGB channels 
            % NOTE: 'lmse.m' only allows a single-channel shading image as an argument.
            [errors(tt).sse, ~, ~, ~] = ...
                lmse(gtdata.albedos_gt(:,:,:,tt), gtdata.illumis_gt(:,:,1,tt), ...
                estdata.albedos_est(:,:,:,tt), estdata.illumis_est(:,:,1,tt), windowSz);
            
            % LMSE that is calulated at each channel, and then averaged over RGB channels
            error_sse_ch = zeros(1,channels);
            for ch=1:channels,
                [error_sse_ch(ch), ~, ~] = lmse(gtdata.albedos_gt(:,:,ch,tt), gtdata.illumis_gt(:,:,ch,tt), ...
                    estdata.albedos_est(:,:,ch,tt), estdata.illumis_est(:,:,ch,tt), windowSz);
            end
            errors(tt).sse_cmean = mean(error_sse_ch);
        end
        
        sses       = [errors(:).sse];
        sse_cmeans = [errors(:).sse_cmean];
        
        fprintf('> LMSE:\n%s\n',num2str(sses,'%3.5f '));
        fprintf('> LMSE(Ch-Mean):\n%s\n',num2str(sse_cmeans,'%3.5f '));
        
        avg_sse       = mean(sses);
        avg_sse_cmean = mean(sse_cmeans);
        
        fprintf('> average LMSE x 1000:\n%3.5f\n', avg_sse*1000);
        fprintf('> average LMSE(Ch-Mean) x 1000:\n%3.5f\n', avg_sse_cmean*1000);
        
        fprintf('\n');
    end

%% EPE averaged over frames

    function [errors_mean, errors] = measureFlow( path_gt, paths_my )        
        S = load(path_gt);
        flows_gt = S.flows_gt;
        occlus_gt = logical(S.occlus_gt);
        
        fprintf('--- TempIncohr: [gt] %s\n', path_gt);
        
        errors_mean = [];
        for ii = 1:length(paths_my),
            
            fprintf('[result %d] %s\n', ii, paths_my{ii});
            
            S = load([paths_my{ii}]);     
            flows_est = S.flows_est_brt;
            
            errors = repmat(struct('aepe',nan,'aepe_mask',nan), [1 size(flows_est,4)]);
            for t=1:size(flows_est,4),
                [~,~,errors(t).aepe] = flowAngErrUV(flows_gt(:,:,:,t), flows_est(:,:,:,t), 0);
                [~,~,errors(t).aepe_mask] = flowAngErrUV(flows_gt(:,:,:,t), flows_est(:,:,:,t), 0, occlus_gt(:,:,:,t));
            end
            errors_mean(end+1).aepe = mean([errors(:).aepe]); 
            errors_mean(end).aepe_mask = mean([errors(:).aepe_mask]);

            fprintf('> EPE:\n%s\n', num2str([errors(:).aepe], '%3.5f '));
            fprintf('> EPE excl. occlusion:\n%s\n', num2str([errors(:).aepe_mask], '%3.5f '));
            fprintf('> average EPE:\n%3.5f\n', errors_mean.aepe);
            fprintf('> average EPE excl. occlusion:\n%3.5f\n', errors_mean.aepe_mask);
            fprintf('\n');
        end
    end
end
